package service

import (
	"github.com/gofiber/fiber/v2"
	"gitea.pena/SQuiz/common/repository/statistics"
	"strconv"
)

type DeviceStatReq struct {
	From uint64 // временные границы выбора статистики
	To   uint64
}

func (s *Service) GetDeviceStatistics(ctx *fiber.Ctx) error {
	quizIDStr := ctx.Params("quizID")

	quizID, err := strconv.ParseInt(quizIDStr, 10, 64)
	if err != nil {
		return ctx.Status(fiber.StatusBadRequest).SendString("Invalid quiz ID format")
	}

	var req DeviceStatReq
	if err := ctx.BodyParser(&req); err != nil {
		return ctx.Status(fiber.StatusBadRequest).SendString("Invalid request data")
	}

	deviceStats, err := s.dal.StatisticsRepo.GetDeviceStatistics(ctx.Context(), statistics.DeviceStatReq{
		QuizId: quizID,
		From:   req.From,
		To:     req.To,
	})
	if err != nil {
		return ctx.Status(fiber.StatusInternalServerError).SendString(err.Error())
	}

	return ctx.Status(fiber.StatusOK).JSON(deviceStats)
}

type GeneralStatsResp struct {
	Open, Result, AvTime, Conversion map[uint64]uint64
}

func (s *Service) GetGeneralStatistics(ctx *fiber.Ctx) error {
	quizIDStr := ctx.Params("quizID")
	quizID, err := strconv.ParseInt(quizIDStr, 10, 64)
	if err != nil {
		return ctx.Status(fiber.StatusBadRequest).SendString("Invalid quiz ID format")
	}

	var req DeviceStatReq
	if err := ctx.BodyParser(&req); err != nil {
		return ctx.Status(fiber.StatusBadRequest).SendString("Invalid request data")
	}

	generalStats, err := s.dal.StatisticsRepo.GetGeneralStatistics(ctx.Context(), statistics.DeviceStatReq{
		QuizId: quizID,
		From:   req.From,
		To:     req.To,
	})
	if err != nil {
		return ctx.Status(fiber.StatusInternalServerError).SendString(err.Error())
	}

	return ctx.Status(fiber.StatusOK).JSON(generalStats)
}

func (s *Service) GetQuestionsStatistics(ctx *fiber.Ctx) error {
	quizIDStr := ctx.Params("quizID")
	quizID, err := strconv.ParseInt(quizIDStr, 0, 64)
	if err != nil {
		return ctx.Status(fiber.StatusBadRequest).SendString("Invalid quiz ID format")
	}

	var req DeviceStatReq
	if err := ctx.BodyParser(&req); err != nil {
		ctx.Status(fiber.StatusBadRequest).SendString("Invalid request data")
	}

	questionsStats, err := s.dal.StatisticsRepo.GetQuestionsStatistics(ctx.Context(), statistics.DeviceStatReq{
		QuizId: quizID,
		From:   req.From,
		To:     req.To,
	})
	if err != nil {
		return ctx.Status(fiber.StatusInternalServerError).SendString(err.Error())
	}

	return ctx.Status(fiber.StatusOK).JSON(questionsStats)
}

type StatisticReq struct {
	From, To uint64 // временные границы выбора статистики
}

func (s *Service) AllServiceStatistics(ctx *fiber.Ctx) error {
	var req StatisticReq
	if err := ctx.BodyParser(&req); err != nil {
		ctx.Status(fiber.StatusBadRequest).SendString("Invalid request data")
	}

	allSvcStats, err := s.dal.StatisticsRepo.AllServiceStatistics(ctx.Context(), req.From, req.To)
	if err != nil {
		return ctx.Status(fiber.StatusInternalServerError).SendString(err.Error())
	}

	return ctx.Status(fiber.StatusOK).JSON(allSvcStats)
}

func (s *Service) GetPipelinesStatistics(ctx *fiber.Ctx) error {
	var req StatisticReq
	if err := ctx.BodyParser(&req); err != nil {
		ctx.Status(fiber.StatusBadRequest).SendString("Invalid request data")
	}

	quizIDStr := ctx.Params("quizID")
	quizID, err := strconv.ParseInt(quizIDStr, 10, 64)
	if err != nil {
		return ctx.Status(fiber.StatusBadRequest).SendString("Invalid quiz ID format")
	}

	result, err := s.chDAL.StatisticClickRepo.GetPipelinesStatistics(ctx.Context(), quizID, req.From, req.To)
	if err != nil {
		return ctx.Status(fiber.StatusInternalServerError).SendString(err.Error())
	}
	return ctx.Status(fiber.StatusOK).JSON(result)
}
